package cmd_handle

import (
	"acme-mana/src/common"
	"acme-mana/src/server"
	"log"
	"os"
	"os/exec"
	"path"
	"path/filepath"
	"strconv"
	"syscall"
)

var stdout *os.File
var stderr *os.File

func start() {
	initLog()

	isDaemon := os.Getenv("GO_DAEMON")
	log.Println("守护进程启动: " + isDaemon)

	if isDaemon != "1" {
		daemonStart()
		return
	}
	if isRunning() {
		log.Println("守护进程已启动, 跳过执行")
		return
	}
	workPath, err := os.Executable()
	if err != nil {
		log.Fatalf("无法获取当前工作路径: %v", err)
	}
	cmd := exec.Cmd{
		Path: workPath,
		Args: os.Args,
		Dir:  filepath.Dir(workPath),
		Env:  append(os.Environ(), "GO_DAEMON=1"),
		//Stdin:       os.Stdin,
		Stdout:      stdout,
		Stderr:      stderr,
		SysProcAttr: &syscall.SysProcAttr{},
	}
	log.Println("启动守护进程中...")
	err = cmd.Start()
	if err != nil {
		log.Fatalf("无法启动守护进程: %v", err)
	}
	err = os.WriteFile(common.PidFile, []byte(strconv.Itoa(cmd.Process.Pid)), 0644)
	if err != nil {
		log.Fatalf("无法写出PID文件: %v", err)
	}
	log.Printf("启动进程启动成功, PID: %d", cmd.Process.Pid)
	os.Exit(0)
}

func daemonStart() {
	// 启动 HttpServer
	server.HttpInstance.Init()
	server.HttpInstance.Start()

	// 启动 SocketServer
	server.SocketInstance.Start()

	// TODO 启动 AutoRefreshCert
}

func isRunning() bool {
	log.Println("Checking if daemon is running...")
	pid, err := readPID()
	if err != nil {
		return false
	}

	process, err := os.FindProcess(pid)
	log.Println("Found process:", process)
	if err != nil {
		log.Println("Failed to find process:", err)
		return false
	}
	return true
}

// 读取PID文件
func readPID() (int, error) {
	log.Println("Reading PID file...")
	data, err := os.ReadFile(common.PidFile)
	if err != nil {
		log.Println("Failed to read PID file:", err)
		return 0, err
	}
	log.Println("PID file content:", string(data))

	pid, err := strconv.Atoi(string(data))
	if err != nil {
		log.Println("Failed to parse PID:", err)
		return 0, err
	}
	log.Println("PID:", pid)

	return pid, nil
}

func initLog() {
	pwd, err := os.Getwd()
	if err != nil {
		log.Fatalf("无法获取当前工作路径: %v", err)
	}
	dir := path.Join(pwd, "log")
	_, err = os.Stat(dir)
	if os.IsNotExist(err) {
		err := os.Mkdir(dir, 0777)
		if err != nil {
			log.Fatalf("无法创建日志目录: %v", err)
		}
	}
	outFile, err := os.OpenFile(path.Join(dir, "out.log"), os.O_CREATE|os.O_APPEND|os.O_WRONLY, 0644)
	defer outFile.Close()
	stdout = outFile

	errFile, err := os.OpenFile(path.Join(dir, "err.log"), os.O_CREATE|os.O_APPEND|os.O_WRONLY, 0644)
	defer stderr.Close()
	stderr = errFile
}
